#define _XOPEN_SOURCE 500
#include <ruby.h>
#include <stdlib.h>
#include <errno.h>
#include <ftw.h>
#include <limits.h>

VALUE rb_cDir;
void Init_dir_size(void);

static __thread unsigned long size = 0;

static int callback(const char *path, const struct stat *st, int typeflag, struct FTW *ftw) {
    if (typeflag == FTW_F || typeflag == FTW_D || typeflag == FTW_SL || typeflag == FTW_SLN)
        size += st->st_size;
    return 0;
}

static int get_size(const char *path, unsigned long *psize) {
    int ret;
    size = 0;
    ret = nftw(path, callback, 10, FTW_PHYS);
    if (ret != -1) *psize = size;
    return ret;
}

static VALUE rb_get_size(VALUE rb_self, VALUE rb_path) {
    unsigned long lsize;
    Check_Type(rb_path, T_STRING);
    if (get_size(StringValueCStr(rb_path), &lsize) == -1)
        rb_raise(rb_eRuntimeError, "Dir.size error: %s", strerror(errno));
    else
        return ULONG2NUM(lsize);
}

void Init_dir_size() {
    rb_cDir = rb_const_get(rb_cObject, rb_intern("Dir"));
    rb_define_singleton_method(rb_cDir, "size", rb_get_size, 1);
}
